/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.sourceforge.basher.*;
import net.sourceforge.basher.events.EventManager;
import net.sourceforge.basher.events.PhaseTransitionEvent;
import net.sourceforge.basher.events.TasksAvailableEvent;
import net.sourceforge.basher.internal.*;
import net.sourceforge.basher.tasks.ATestTask;
import org.apache.commons.logging.Log;
import org.easymock.EasyMock;

import static org.easymock.EasyMock.expect;

/**
 * @author Johan Lindquist
 * @version 1.0
 */
public class TestTaskManagerImpl extends BasherTestCase
{
    private Randomizer _randomizer;
    private Log _log;
    private EventManager _eventManager;
    private BasherContext _basherContext;
    private TaskConfigurationExtractor _taskConfigurationExtractor;

    public void testInitializationEmptyTaskInfoList()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);

        taskManagerImpl.setModuleDescriptorTaskContributions(Collections.<ModuleDescriptorTaskContribution>emptyList());

        _log.warn("No task contributions supplied");
        _log.debug("No task providers available");

        replayAll();

        taskManagerImpl.initializeService();
        verifyAll();


    }

    public void testInitializationNotWired()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();
        initializeControls();
        replayAll();
        try
        {
            taskManagerImpl.initializeService();
        }
        catch (IllegalStateException e)
        {
            assertEquals("no randomizer", e.getMessage());
        }

        try
        {
            taskManagerImpl.setRandomizer(_randomizer);
            taskManagerImpl.initializeService();
        }
        catch (IllegalStateException e)
        {
            assertEquals("no log", e.getMessage());
        }
        verifyAll();
    }

    public void testNotInitialized()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();
        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);
        replayAll();

        try
        {
            taskManagerImpl.addTask((TaskContribution) null);
        }
        catch (IllegalStateException e)
        {
            assertEquals("not initialized", e.getMessage());
        }
        try
        {
            taskManagerImpl.getNextTaskExecutionContext();
        }
        catch (IllegalStateException e)
        {
            assertEquals("not initialized", e.getMessage());
        }
        try
        {
            taskManagerImpl.getNumberOfTasks();
        }
        catch (IllegalStateException e)
        {
            assertEquals("not initialized", e.getMessage());
        }
        try
        {
            taskManagerImpl.getTaskByName("fred");
        }
        catch (IllegalStateException e)
        {
            assertEquals("not initialized", e.getMessage());
        }
        try
        {
            taskManagerImpl.getTasks();
        }
        catch (IllegalStateException e)
        {
            assertEquals("not initialized", e.getMessage());
        }
        try
        {
            taskManagerImpl.removeTaskExecutionContext(null);
        }
        catch (IllegalStateException e)
        {
            assertEquals("not initialized", e.getMessage());
        }

        verifyAll();
    }


    public void testNullAddTask()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);
        _log.warn("No task contributions supplied");
        _log.debug("No task providers available");

        replayAll();
        taskManagerImpl.initializeService();

        try
        {
            taskManagerImpl.addTask((TaskContribution) null);
        }
        catch (NullPointerException e)
        {
            // should happne
            assertEquals("Bad message", "taskContribution", e.getMessage());
        }

    }

    public void testNullSetTaskInfo()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        try
        {
            taskManagerImpl.setModuleDescriptorTaskContributions(null);
        }
        catch (NullPointerException e)
        {
            // should happne
            assertEquals("Bad message", "moduleDescriptorTaskContributions", e.getMessage());
        }

    }

    public void testInitializationTwoItemTaskInfoList()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);

        List<ModuleDescriptorTaskContribution> moduleDescriptorTaskContributions = new ArrayList<ModuleDescriptorTaskContribution>();

        final ATestTask task1 = new ATestTask("task1");

        final ModuleDescriptorTaskContribution taskContribution1 = createModuleDescriptorTaskContributionFromTask(task1);
        moduleDescriptorTaskContributions.add(taskContribution1);

        final ATestTask task2 = new ATestTask("task2");
        final ModuleDescriptorTaskContribution taskContribution2 = createModuleDescriptorTaskContributionFromTask(task2);
        moduleDescriptorTaskContributions.add(taskContribution2);

        taskManagerImpl.setModuleDescriptorTaskContributions(moduleDescriptorTaskContributions);

        _log.debug("No task providers available");

        _log.debug("Task contribution added");
        _log.debug("Task contribution added");

        _log.info("'2' task contribution(s) added");

        replayAll();

        taskManagerImpl.initializeService();

        final Set<TaskContribution> taskContributions = taskManagerImpl.getTasks();

        assertEquals("Wrong number of tasks in list", 2, taskContributions.size());

        assertTrue("task not in list", taskContributions.contains(new TaskContribution(task1,taskContribution1)));
        assertTrue("task not in list", taskContributions.contains(new TaskContribution(task2,taskContribution2)));

        final TaskContribution task1Retrieved = taskManagerImpl.getTaskByName("task1");

        assertEquals("Wrong task retrieved by name", task1, task1Retrieved.getTask());

        final TaskContribution task2Retrieved = taskManagerImpl.getTaskByName("task2");
        assertEquals("Wrong task retrieved by name", task2, task2Retrieved.getTask());
        verifyAll();

    }

    public void testInitializationTwoItemTaskInfoListWithFollower()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);

        final List<ModuleDescriptorTaskContribution> moduleDescriptorTaskContributions = new ArrayList<ModuleDescriptorTaskContribution>();

        final ATestTask task1 = new ATestTask("task1");
        final ModuleDescriptorTaskContribution taskContribution1 = createModuleDescriptorTaskContributionFromTask(task1);
        moduleDescriptorTaskContributions.add(taskContribution1);

        final ATestTask task2 = new ATestTask("task2");

        final ModuleDescriptorTaskContribution taskContribution2 = createModuleDescriptorTaskContributionFromTask(task2);
        moduleDescriptorTaskContributions.add(taskContribution2);

        // final ATestTask task2Follower = new ATestTask("task2Follower");

        taskManagerImpl.setModuleDescriptorTaskContributions(moduleDescriptorTaskContributions);

        _log.debug("No task providers available");

        _log.debug("Task contribution added");
        _log.debug("Task contribution added");
        // _log.debug("Task 'task2' added - 1 follower(s)");

        _log.info("'2' task contribution(s) added");

        replayAll();

        taskManagerImpl.initializeService();

        final Set<TaskContribution> tasks = taskManagerImpl.getTasks();

        assertEquals("Wrong number of tasks in list", 2, tasks.size());


        assertTrue("task not in list", tasks.contains(new TaskContribution(task1,taskContribution1)));
        assertTrue("task not in list", tasks.contains(new TaskContribution(task2,taskContribution2)));

        final TaskContribution task1Retrieved = taskManagerImpl.getTaskByName("task1");

        assertEquals("Wrong task retrieved by name", task1, task1Retrieved.getTask());

        final TaskContribution task2Retrieved = taskManagerImpl.getTaskByName("task2");
        assertEquals("Wrong task retrieved by name", task2, task2Retrieved.getTask());

//        assertEquals("Wrong number of followers", 0, task1Retrieved.getFollowers().size());
//        assertEquals("Wrong number of followers", 1, task2Retrieved.getFollowers().size());

        verifyAll();

    }


    public void testGetRandomTask()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);

        final List<ModuleDescriptorTaskContribution> moduleDescriptorTaskContributions = new ArrayList<ModuleDescriptorTaskContribution>();

        final ATestTask task1 = new ATestTask("task1");
        final ModuleDescriptorTaskContribution taskContribution1 = createModuleDescriptorTaskContributionFromTask(task1);
        moduleDescriptorTaskContributions.add(taskContribution1);

        final ATestTask task2 = new ATestTask("task2");

        final ModuleDescriptorTaskContribution taskContribution2 = createModuleDescriptorTaskContributionFromTask(task2);
        moduleDescriptorTaskContributions.add(taskContribution2);


        taskManagerImpl.setModuleDescriptorTaskContributions(moduleDescriptorTaskContributions);

        _log.debug("No task providers available");
        expect(_log.isDebugEnabled()).andReturn(false).times(2);

        _log.debug("Task contribution added");
        _log.debug("Task contribution added");

        _log.info("'2' task contribution(s) added");

        trainPrepareForPhase();

        expect(_randomizer.getRandomInt(2)).andReturn(1);

        expect(_randomizer.getRandomInt(2)).andReturn(0);

        replayAll();

        taskManagerImpl.initializeService();

        taskManagerImpl.basherEvent(new PhaseTransitionEvent(new BasherContext(), Phase.WARMUP, Phase.RUN, 0));

        TaskExecutionContext taskExecution2Context = taskManagerImpl.getNextTaskExecutionContext();
        Task task2Retrieved = taskExecution2Context.getTask();

        assertEquals("Wrong task retrived by random", task2, task2Retrieved);

        TaskExecutionContext taskExecution1Context = taskManagerImpl.getNextTaskExecutionContext();
        Task task1Retrieved = taskExecution1Context.getTask();
        assertEquals("Wrong task retrived by random", task1, task1Retrieved);
        verifyAll();


    }

    private void trainPrepareForPhase()
    {
        expect(_log.isDebugEnabled()).andReturn(false).times(1);
        _log.info("Preparing available tasks for phase 'RUN'");
        _log.info("Number available tasks: 2");
        _log.debug("Filtering tasks for phase 'RUN'");
        _log.debug("Filtering according to includes/excludes");
        _log.debug("Sorting tasks according to task name");
        // _log.debug("Resetting available tasks");
        _log.info("Preparation of tasks for phase 'RUN' completed");
        _log.info("Number available tasks after preparation: 2");
    }

    public void testGetByName()
    {
        final TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);

        final List<ModuleDescriptorTaskContribution> moduleDescriptorTaskContributions = new ArrayList<ModuleDescriptorTaskContribution>();

        final ATestTask task1 = new ATestTask("task1");
        final ModuleDescriptorTaskContribution taskContribution1 = createModuleDescriptorTaskContributionFromTask(task1);
        moduleDescriptorTaskContributions.add(taskContribution1);

        final ATestTask task2 = new ATestTask("task2");

        final ModuleDescriptorTaskContribution taskContribution2 = createModuleDescriptorTaskContributionFromTask(task2);
        moduleDescriptorTaskContributions.add(taskContribution2);


        taskManagerImpl.setModuleDescriptorTaskContributions(moduleDescriptorTaskContributions);

        _log.debug("No task providers available");
        expect(_log.isDebugEnabled()).andReturn(false).times(2);

        _log.debug("Task contribution added");
        _log.debug("Task contribution added");

        _log.info("'2' task contribution(s) added");

/*
        _log.info("Removing task 'task2' from active list");
        _log.info("Remaining number of tasks: 1");

*/
        trainPrepareForPhase();

        replayAll();

        taskManagerImpl.initializeService();
        taskManagerImpl.prepareTasksForPhase(Phase.RUN, _basherContext);

        final TaskContribution task1Retrieved = taskManagerImpl.getTaskByName("task1");
        assertEquals("Wrong task retrived by name", task1, task1Retrieved.getTask());

        final TaskContribution task2Retrieved = taskManagerImpl.getTaskByName("task2");
        assertEquals("Wrong task retrived by name", task2, task2Retrieved.getTask());

        assertNull("could retrieve non-existant task by name", taskManagerImpl.getTaskByName("not-there"));


        //taskManagerImpl.removeTask(task2);
        //assertNull("could retrieve non-existant task by name", taskManagerImpl.getTaskByName("not-there"));

        verifyAll();

    }


    public void testAddTask()
    {
        final TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);

        final List<ModuleDescriptorTaskContribution> moduleDesriptorTaskContributions = new ArrayList<ModuleDescriptorTaskContribution>();

        final ATestTask task1 = new ATestTask("task1");
        moduleDesriptorTaskContributions.add(createModuleDescriptorTaskContributionFromTask(task1));

        taskManagerImpl.setModuleDescriptorTaskContributions(moduleDesriptorTaskContributions);

        _log.debug("No task providers available");
        expect(_log.isDebugEnabled()).andReturn(false).times(2);

        _log.debug("Task contribution added");

        _log.info("'1' task contribution(s) added");
        // _log.info("Task 'task2' added");

        _eventManager.publish(EasyMock.isA(TasksAvailableEvent.class));

        expect(_randomizer.getRandomInt(2)).andReturn(1);

        trainPrepareForPhase();

        replayAll();

        taskManagerImpl.initializeService();

        final ATestTask task2 = new ATestTask("task2");

        final TaskContribution contribution = new TaskContribution(task2);
        contribution.setTaskName(task2.getName());
        taskManagerImpl.addTask(contribution);

        taskManagerImpl.prepareTasksForPhase(Phase.RUN, _basherContext);


        TaskExecutionContext taskExecutionContext = taskManagerImpl.getNextTaskExecutionContext();
        Task task2Retrieved = taskExecutionContext.getTask();

        assertEquals("Wrong task retrived by random", task2, task2Retrieved);

        Set<TaskContribution> tasks = taskManagerImpl.getTasks();
        assertEquals("Wrong number of active tasks", 2, tasks.size());
        assertEquals("Wrong number of active tasks", 2, taskManagerImpl.getNumberOfTasks());

        task2Retrieved = taskManagerImpl.getTaskByName("task2").getTask();
        assertEquals("Wrong task retrived by name", task2, task2Retrieved);

        verifyAll();

    }

    public void testRemoveTaskExecutionContext()
    {



    }

    private ModuleDescriptorTaskContribution createModuleDescriptorTaskContributionFromTask(final ATestTask task)
    {
        final ModuleDescriptorTaskContribution moduleDescriptorTaskContribution = new ModuleDescriptorTaskContribution(task);
        moduleDescriptorTaskContribution.setTaskName(task.getName());
        moduleDescriptorTaskContribution.addApplicablePhase(Phase.RUN);
        return moduleDescriptorTaskContribution;
    }

    public void testGetNextTaskExecutionContext()
    {
        TaskManagerImpl taskManagerImpl = new TaskManagerImpl();

        initializeControls();
        initializeTaskManager(taskManagerImpl, _log, _randomizer, _eventManager, _taskConfigurationExtractor);

        _log.debug("No task providers available");

        _log.warn("No task contributions supplied");
        _log.debug("No active tasks available");

        replayAll();

        taskManagerImpl.initializeService();

        TaskExecutionContext taskExecutionContext = taskManagerImpl.getNextTaskExecutionContext();
        assertNull("Unexpected task execution returned by random", taskExecutionContext);
        verifyAll();

    }


    private void initializeControls()
    {

        _log = createMock(Log.class);
        _randomizer = createMock(Randomizer.class);
        _eventManager = createMock(EventManager.class);
        _taskConfigurationExtractor = createMock(TaskConfigurationExtractor.class);
    }

    private void initializeTaskManager(final TaskManagerImpl taskManagerImpl, final Log log, final Randomizer randomizer, final EventManager eventManager, final TaskConfigurationExtractor taskConfigurationExtractor)
    {
        taskManagerImpl.setLog(log);
        taskManagerImpl.setRandomizer(randomizer);
        taskManagerImpl.setEventManager(eventManager);
        taskManagerImpl.setTaskConfigurationExtractor(taskConfigurationExtractor);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        _basherContext = new BasherContext();
    }
}
