/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sourceforge.basher.internal.impl;

import java.util.ArrayList;
import java.util.UUID;

import net.sourceforge.basher.*;
import net.sourceforge.basher.internal.Randomizer;
import net.sourceforge.basher.internal.TimeSource;
import net.sourceforge.basher.tasks.ATestTask;
import org.apache.commons.logging.Log;
import org.easymock.classextension.EasyMock;

import static org.easymock.EasyMock.expect;

/**
 * @author Johan Lindquist
 * @version 1.0
 */
public class TestTaskInvokerImpl extends BasherTestCase
{
    private Collector _collector;
    private Log _log;
    private Randomizer _randomizer;
    private TaskManager _taskManager;
    private TimeSource _timeSource;

    private UUID _identifier1;
    private UUID _identifier2;

    public void testNotInvokeWeight()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 50, 1 );
        trainGetElapsedTime( 10L, 1 );

        trainDebugLogStatement( "Not running task: net.sourceforge.basher.tasks.ATestTask (weight: 50 / weigher: 50)", 1 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 0, 50, Long.MAX_VALUE, 0 ) );

        _collector.notRun( taskExecutionContext, 0, 0 );

        replayAll();


        taskInvokerImpl.invokeTask( taskExecutionContext );
        verifyAll();


    }

    private void trainGetNanoTime( final int times )
    {
        expect( _timeSource.getNanoTime() ).andReturn( 0L ).times( times );
    }

    public void testInvokeWeight()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 1 );

        trainGetElapsedTime( 10L, 1 );

        trainGetCurrentTime( 1, 2 );

        trainGetNanoTime( 2 );

        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask", 1 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskConfiguration taskConfiguration = createTaskConfiguration( ATestTask.NAME, 0, 100, Long.MAX_VALUE, 0 );
        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, taskConfiguration );

        _collector.success( taskExecutionContext, 0, 0 );

        replayAll();

        taskInvokerImpl.invokeTask( taskExecutionContext );

        verifyAll();


    }

    private void trainGetElapsedTime( final long elapsedTime, final int times )
    {
        expect( _timeSource.getElapsedTime() ).andReturn( elapsedTime ).times( times );
    }

    private void trainGetCurrentTime( final long value, final int times )
    {
        expect( _timeSource.getCurrentTime() ).andReturn( value ).times( times );
    }

    public void testNotInvokeBeforeStartTime()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetElapsedTime( 10L, 1 );

        trainDebugLogStatement( "Not running task: net.sourceforge.basher.tasks.ATestTask (not within time boundary 0/100/9223372036854775807)", 1 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 0, 100, Long.MAX_VALUE, 100 ) );

        _collector.notRun( taskExecutionContext, 0, 0 );

        replayAll();


        taskInvokerImpl.invokeTask( taskExecutionContext );
        verifyAll();


    }

    private void trainDebugLogStatement( final String message, final int times )
    {
        expect( _log.isDebugEnabled() ).andReturn( true ).times( times );
        _log.debug( message );
        if ( times > 1 )
        {
            EasyMock.expectLastCall().times( times );
        }
    }

    private void trainDebugLogStatement( final String message )
    {
        trainDebugLogStatement( message, 1 );
    }

    public void testNotInvokeAfterEndTime()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetElapsedTime( 10000L, 1 );

        trainDebugLogStatement( "Not running task: net.sourceforge.basher.tasks.ATestTask (not within time boundary 10/0/1)", 1 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskExecutionContext context = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 0, 100, 1, 0 ) );
        _collector.notRun( context, 0, 0 );

        replayAll();

        taskInvokerImpl.invokeTask( context );

        verifyAll();

    }

    public void testInvokeAfterStartTime()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 1 );

        trainGetElapsedTime( 11000L, 1 );
        trainGetCurrentTime( 1, 2 );

        trainGetNanoTime( 2 );

        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask", 1 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();



        final TaskConfiguration taskConfiguration = createTaskConfiguration( ATestTask.NAME, 0, 100, Long.MAX_VALUE, 10 );
        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, taskConfiguration );

        _collector.success( taskExecutionContext, 0, 0 );

        replayAll();

        taskInvokerImpl.invokeTask( taskExecutionContext );

        verifyAll();

    }

    public void testInvokeBeforeEndTime()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 1 );

        trainGetElapsedTime( 1L, 1 );
        trainGetCurrentTime( 1, 2 );

        trainGetNanoTime( 2 );

        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask", 1 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 0, 100, 1, 0 ) );

        _collector.success( taskExecutionContext, 0, 0 );

        replayAll();


        taskInvokerImpl.invokeTask( taskExecutionContext );
        verifyAll();


    }


    public void testInvokeLessThanMaxInvocations()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 2 );

        trainGetElapsedTime( 1L, 2 );
        trainGetCurrentTime( 1, 4 );

        trainGetNanoTime( 4 );

        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask", 2 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 2, 100, Long.MAX_VALUE, 0 ) );
        _collector.success( taskExecutionContext, 0, 0 );
        _collector.success( taskExecutionContext, 0, 0 );

        replayAll();


        taskInvokerImpl.invokeTask( taskExecutionContext );
        taskInvokerImpl.invokeTask( taskExecutionContext );
        verifyAll();


    }

    private void trainGetRandomInt( final int range, final int value, final int times )
    {
        expect( _randomizer.getRandomInt( range ) ).andReturn( value ).times( times );
    }

    public void testInvokeUnlimitedMaxInvocations()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 2 );

        trainGetElapsedTime( 1L, 2 );
        trainGetCurrentTime( 1, 4 );

        trainGetNanoTime( 4 );

        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask", 2 );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 0, 100, Long.MAX_VALUE, 0 ) );

        _collector.success( taskExecutionContext, 0, 0 );
        _collector.success( taskExecutionContext, 0, 0 );

        replayAll();


        taskInvokerImpl.invokeTask( taskExecutionContext );
        taskInvokerImpl.invokeTask( taskExecutionContext );

        verifyAll();

    }


    public void testNotInvokeLessThanMaxInvocations()
    {

        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 2 );

        trainGetElapsedTime( 1L, 2 );
        trainGetCurrentTime( 1, 4 );

        trainGetNanoTime( 4 );

        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask", 2 );
        trainDebugLogStatement( "Not running task: net.sourceforge.basher.tasks.ATestTask (max invocations reached)" );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final Task task = new ATestTask();

        final TaskExecutionContext context = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 2, 100, Long.MAX_VALUE, 0 ) );

        _collector.success( context, 0, 0 );
        _collector.success( context, 0, 0 );
        _collector.notRun( context, 0, 0 );

        _taskManager.removeTaskExecutionContext( _identifier1 );

        replayAll();


        taskInvokerImpl.invokeTask( context );
        taskInvokerImpl.invokeTask( context );
        taskInvokerImpl.invokeTask( context );

        verifyAll();

    }

    private TaskConfiguration createTaskConfiguration( final String name, final int maxInvocations, final int weight, final long stopAfter, final long runFrom )
    {
        final TaskConfiguration configuration = new TaskConfiguration();
        configuration.setMaxInvocations( maxInvocations );
        configuration.setTaskName( name );
        configuration.setWeight( weight );
        configuration.setRunFrom( runFrom );
        configuration.setStopAfter( stopAfter );
        return configuration;
    }

    public void testNotRun()
    {
        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 1 );

        trainGetElapsedTime( 1L, 1 );
        trainGetCurrentTime( 1, 2 );

        trainGetNanoTime( 2 );

        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask" );
        trainDebugLogStatement( "Task not run: net.sourceforge.basher.tasks.ATestTask (forced not run)" );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final ATestTask task = new ATestTask();
        task.setDontRun( true );

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 0, 100, Long.MAX_VALUE, 0 ) );
        _collector.notRun( taskExecutionContext, 0, 0 );

        replayAll();

        taskInvokerImpl.invokeTask( taskExecutionContext );

        verifyAll();

    }

    public void testFailed()
    {
        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 1 );

        trainGetElapsedTime( 1L, 1 );
        trainGetCurrentTime( 1, 2 );
        trainGetNanoTime( 2 );


        trainDebugLogStatement( "Invoking task: net.sourceforge.basher.tasks.ATestTask" );

        final TaskFailedException taskFailedException = new TaskFailedException( "forced failure", ATestTask.EXCEPTION );
        expect( _log.isErrorEnabled() ).andReturn( true );
        _log.error( "Task failed: net.sourceforge.basher.tasks.ATestTask", taskFailedException );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final ATestTask task = new ATestTask();
        task.setFailWithError( true );

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( ATestTask.NAME, 0, 100, Long.MAX_VALUE, 0 ) );

        _collector.fail( taskExecutionContext, 0, 0, taskFailedException );

        replayAll();

        taskInvokerImpl.invokeTask( taskExecutionContext );

        verifyAll();

    }

    public void testInvokeFollowers()
    {
        final TaskInvokerImpl taskInvokerImpl = new TaskInvokerImpl();

        initializeControls();

        trainGetRandomInt( 100, 40, 2 );
        trainGetElapsedTime( 1L, 2 );
        trainGetCurrentTime( 1, 4 );

        trainGetNanoTime( 4 );

        trainDebugLogStatement( "Invoking task: task1" );
        trainDebugLogStatement( "Invoking followers" );
        trainDebugLogStatement( "Invoking task: follower" );

        initializeTaskInvoker( taskInvokerImpl, _collector, _log, _randomizer, _taskManager, _timeSource );

        final ATestTask task = new ATestTask( "task1" );
        final ATestTask followingTask = new ATestTask( "follower" );

        final TaskExecutionContext taskExecutionContext = new TaskExecutionContext( _identifier1, task, createTaskConfiguration( "task1", 0, 100, Long.MAX_VALUE, 0 ) );
        final TaskExecutionContext taskExecutionContextFollower = new TaskExecutionContext( _identifier2, followingTask, createTaskConfiguration( "follower", 0, 100, Long.MAX_VALUE, 0 ) );
        taskExecutionContext.addFollower( taskExecutionContextFollower );

        _collector.success( taskExecutionContext, 0, 0 );
        _collector.success( taskExecutionContextFollower, 0, 0 );

        replayAll();


        taskInvokerImpl.invokeTask( taskExecutionContext );

        verifyAll();

    }

    private void initializeControls()
    {
        _collector = createMock( Collector.class );
        _log = createMock( Log.class );
        _randomizer = createMock( Randomizer.class );
        _taskManager = createMock( TaskManager.class );
        _timeSource = createMock( TimeSource.class );
    }

    private void initializeTaskInvoker( final TaskInvokerImpl taskInvokerImpl, final Collector collector, final Log log, final Randomizer randomizer, final TaskManager taskManager, final TimeSource timeSource )
    {
        taskInvokerImpl.setCollector( collector );
        taskInvokerImpl.setLog( log );
        taskInvokerImpl.setRandomizer( randomizer );
        taskInvokerImpl.setTaskManager( taskManager );
        taskInvokerImpl.setTimeSource( timeSource );
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        _identifier1 = UUID.randomUUID();
        _identifier2 = UUID.randomUUID();
    }
}
